/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdambalance.h"


static int         gdam_balance_read               (GdamSource*	  gain,
                                       	         GdamSample*	  buf,
				       	         int		  n);

static void        gdam_balance_class_init         (GdamFilterClass* klass);
static void        gdam_balance_init               (GdamBalance*      gain);

static GtkObjectClass* parent_class;

GtkType gdam_balance_get_type()
{
	static GtkType gain_type = 0;
	if (!gain_type) {
		static const GtkTypeInfo gain_info = {
			"GdamBalance",
			sizeof(GdamBalance),
			sizeof(GdamFilterClass),
			(GtkClassInitFunc) gdam_balance_class_init,
			(GtkObjectInitFunc) gdam_balance_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		parent_class = gtk_type_class(GDAM_TYPE_FILTER);
		gain_type = gtk_type_unique(GDAM_TYPE_FILTER, &gain_info);
	}
	return gain_type;
}
static void gdam_balance_arg_set_volumes(GtkObject* object, GdamArg* arg)
{
	gdouble* gain_array = GDAM_ARG_DOUBLE_ARRAY(arg).array;
	GdamBalance* balance = GDAM_BALANCE(object);
	if (GDAM_ARG_DOUBLE_ARRAY(arg).length != 4) {
		gdam_debug("set_volumes: array must be of length 4");
		return;
	}
	balance->volumes[0][0] = (int)(gain_array[0] * GdamFixed_ONE);
	balance->volumes[0][1] = (int)(gain_array[1] * GdamFixed_ONE);
	balance->volumes[1][0] = (int)(gain_array[2] * GdamFixed_ONE);
	balance->volumes[1][1] = (int)(gain_array[3] * GdamFixed_ONE);
}
static void gdam_balance_arg_get_volumes(GtkObject* object, GdamArg* arg)
{
	gdouble* array = g_new(gdouble, 4);
	GdamBalance* balance = GDAM_BALANCE(object);
	GDAM_ARG_DOUBLE_ARRAY(arg).length = 4;
	GDAM_ARG_DOUBLE_ARRAY(arg).array = array;
	array[0] = (gdouble)balance->volumes[0][0] / GdamFixed_ONE;
	array[1] = (gdouble)balance->volumes[0][1] / GdamFixed_ONE;
	array[2] = (gdouble)balance->volumes[1][0] / GdamFixed_ONE;
	array[3] = (gdouble)balance->volumes[1][1] / GdamFixed_ONE;
}

static void gdam_balance_class_init(GdamFilterClass* klass)
{
	GdamSourceClass* source_class = (GdamSourceClass*)klass;
	source_class->read = gdam_balance_read;

	gdam_arg_register(GTK_OBJECT_CLASS(klass),
	                  "volumes",
			  GDAM_ARG_TYPE_DOUBLE_ARRAY,
			  gdam_balance_arg_set_volumes,
			  gdam_balance_arg_get_volumes,
			  GDAM_ARG_VITAL);
}

static void gdam_balance_init(GdamBalance* gain)
{
	gain->volumes[0][0] = GdamFixed_ONE;
	gain->volumes[1][1] = GdamFixed_ONE;
}

static int gdam_balance_read(GdamSource* source, GdamSample* buf, int n)
{
	GdamBalance* balance = GDAM_BALANCE(source);
	GdamFilter* filter = GDAM_FILTER(source);
	GdamSource* filtered;
	int num_read, i;
	/* vol_io == i => left/right input channel src,
	             o => left/right output channel dest. */
	GdamFixed vol_ll = balance->volumes[0][0];
	GdamFixed vol_lr = balance->volumes[0][1];
	GdamFixed vol_rl = balance->volumes[1][0];
	GdamFixed vol_rr = balance->volumes[1][1];
	filtered = filter->filtered;
	if (!filtered)
		return 0;
	num_read = gdam_source_read(filtered, buf, n);
	for (i = 0; i < num_read; i++) {
		int left, right;
		int in_left = buf[i].s[0];
		int in_right = buf[i].s[1];
		left  = ( ((int)((vol_ll>>3) * in_left ))
		        + ((int)((vol_rl>>3) * in_right)) ) >> 13;
		right = ( ((int)((vol_lr>>3) * in_left ))
		        + ((int)((vol_rr>>3) * in_right)) ) >> 13;
		buf[i].s[0] = CLAMP(left,  GDAM_SAMPLE_MIN, GDAM_SAMPLE_MAX);
		buf[i].s[1] = CLAMP(right, GDAM_SAMPLE_MIN, GDAM_SAMPLE_MAX);
	}
	return num_read;
}
