/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamsynthonefir.h"
#include <gdam/gdamplugin.h>

static int gdam_synth_one_fir_synthesize(GdamSynthSource* source,
                                         gdouble*         buf,
					 int              num)
{
	GdamSynthOneFir* one_fir = GDAM_SYNTH_ONE_FIR(source);
	GdamSynthSource* filtered = ((GdamSynthFilter*)one_fir)->filtered;
	double coeff0 = one_fir->coeff[0];
	double coeff1 = one_fir->coeff[1];
	double last = one_fir->last_sample;
	int num_read = gdam_synth_source_synthesize(filtered, buf, num);
	num = num_read;
	while (num_read-- > 0) {
		gdouble output;
		gdouble cur = *buf;
		output = coeff0 * cur + coeff1 * last;
		last = cur;
		*buf++ = output;
	}
	one_fir->last_sample = last;
	return num;
}

static void gdam_synth_one_fir_init(GdamSynthOneFir* synth_one_fir)
{

}
static void gdam_synth_one_fir_class_init(GdamSynthOneFirClass* klass)
{
	GDAM_SYNTH_SOURCE_CLASS(klass)->synthesize
					= gdam_synth_one_fir_synthesize;
}

GtkType gdam_synth_one_fir_get_type()
{
	static GtkType synth_one_fir_type = 0;
	if (!synth_one_fir_type) {
		GtkType parent_type;
		static const GtkTypeInfo synth_one_fir_info = {
			"GdamSynthOneFir",
			sizeof(GdamSynthOneFir),
			sizeof(GdamSynthOneFirClass),
			(GtkClassInitFunc) gdam_synth_one_fir_class_init,
			(GtkObjectInitFunc) gdam_synth_one_fir_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		parent_type = gdam_plugin_get_type_by_name("server",
		                                           "GdamSynthFilter");
		synth_one_fir_type = gtk_type_unique(parent_type,
		                                 &synth_one_fir_info);
	}
	return synth_one_fir_type;
}
